use std::{
    fmt,
    sync::{MutexGuard, PoisonError},
};

use hidapi::HidError;

use super::TouchbarMode;

#[derive(Debug)]
pub enum TouchbarError {
    Init(HidError),
    PoisionCantChangeMode,
    PoisionCantSendEvent,
    AlreadyListening,
}

impl fmt::Display for TouchbarError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            TouchbarError::PoisionCantChangeMode => write!(
                f,
                "Unable to Change Mode, because previous thread was posioned"
            ),
            TouchbarError::PoisionCantSendEvent => {
                write!(f, "Unable to Send Event, previous thread was poisioned")
            }
            TouchbarError::AlreadyListening => {
                write!(f, "There is Allready a Listenr Thread running.")
            }
            TouchbarError::Init(hid_err) => hid_err.fmt(f),
        }
    }
}

impl std::error::Error for TouchbarError {}

impl From<PoisonError<MutexGuard<'_, TouchbarMode>>> for TouchbarError {
    fn from(_: PoisonError<MutexGuard<TouchbarMode>>) -> Self {
        Self::PoisionCantChangeMode
    }
}
impl From<PoisonError<MutexGuard<'_, bool>>> for TouchbarError {
    fn from(_: PoisonError<MutexGuard<bool>>) -> Self {
        Self::PoisionCantSendEvent
    }
}

impl From<HidError> for TouchbarError {
    fn from(err: HidError) -> Self {
        TouchbarError::Init(err)
    }
}
