mod err;
mod imp;

pub use err::*;
pub use imp::*;

#[derive(Clone, Copy)]
pub enum TouchbarMode {
    EscOnly = 0x00,
    Function = 0x01,
    Special = 0x02,
    Off = 0x03,
}

#[derive(Clone, Copy)]
pub enum TouchbarEvent {
    Esc,
    Function(TouchbarFunctionEvent),
    BrightnessDown,
    BrightnessUp,
    BacklightDown,
    BacklightUp,
    Media(TouchbarMediaEvent),
}

#[derive(Clone, Copy)]
pub enum TouchbarFunctionEvent {
    F1,
    F2,
    F3,
    F4,
    F5,
    F6,
    F7,
    F8,
    F9,
    F10,
    F11,
    F12,
}

#[derive(Clone, Copy)]
pub enum TouchbarMediaEvent {
    SkipBack,
    PauseResume,
    SkipForward,
    MuteVolume,
    VolumeDown,
    VolumeUp,
}

pub trait TouchbarEventListener: Send {
    fn on_event(&self, event: TouchbarEvent);
}

pub trait Touchbar {
    fn set_mode(&mut self, mode: TouchbarMode) -> Result<(), TouchbarError>;
    fn add_listener(&mut self, listener: Box<dyn TouchbarEventListener>);
    fn start_listening_input(&mut self) -> Result<(), TouchbarError>;
    fn stop_listening_input(&mut self) -> Result<(), TouchbarError>;
}
